/* This file is part of turtle, a turtle-graphics system for Scheme.

Copyright (C) 2008  Luca Saiu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */


#ifndef GRAPHICS_H_
#define GRAPHICS_H_

/* A color is defined by the three RGB components, expressed
   as floats, in the interval [0.0, 1.0]: */
struct turtle_color{
  float red;
  float green;
  float blue;
}; // struct

/* We always refer to a color using a pointer: */
typedef struct turtle_color* turtle_color_t;

/* Get the current window size: */
int turtle_get_window_width(void);
int turtle_get_window_height(void);

/* Initialize the graphics subsystem, making a new window of the given size,
   or fail with a fatal error: */
void turtle_initialize_graphics(const int width, const int height);

/* Finalize the graphics subsystem: */
void turtle_finalize_graphics(void);

/* Clear the whole window by overdrawing with the given color: */
void turtle_clear_window(const turtle_color_t color);

/* Draw a pixel on the window, at the given coordinates, with the given color,
   and show it immediately: */
void turtle_draw_pixel(const int x, const int y,
                       const turtle_color_t color);

/* Draw a pixel which is part of the cursor: */
void turtle_draw_cursor_pixel(const int x, const int y, const turtle_color_t color);

/* Draw a line on the window, at the given position, with the given color,
   and show it immediately. Both extreme points are included. */
void turtle_draw_line(const int x1, const int y1, // position of the first point
                      const int x2, const int y2, // position of the last point
                      const turtle_color_t color);

/* Draw a line which is part of the cursor: */
void turtle_draw_cursor_line(const int x1, const int y1, // position of the first point
                             const int x2, const int y2, // position of the last point
                             const turtle_color_t color);

/* Redraw the screen, so that any change becomes visible (and the
   window is shown again after a portion of it is exposed by another
   window which has been moved, destroyed or resized). This does *not*
   draw the cursor: */
void turtle_refresh(void);

/* Just like turtle_refresh(), but this also shows the cursor. This function
   is quite expensive in its current implementation: */
void turtle_refresh_with_cursor(void);

/* Manage events like expose. This must be called periodically. */
void turtle_manage_events_if_any(void);

/* Replace the current handler (if any) for the expose event with the given
   function.
   The function will be called by turtle_manage_events_if_any() if the
   window is exposed: */
void turtle_set_expose_event_handler(void (*function)(void));

/* Some predefined colors: */
extern const turtle_color_t turtle_black;
extern const turtle_color_t turtle_white;
extern const turtle_color_t turtle_red;
extern const turtle_color_t turtle_green;
extern const turtle_color_t turtle_blue;
extern const turtle_color_t turtle_cyan;
extern const turtle_color_t turtle_magenta;
extern const turtle_color_t turtle_yellow;

#endif // #ifndef GRAPHICS_H_
